# -*- coding: utf-8 -*-

from django.test import TestCase
from cjkcms.templatetags.gravatar import gravatar_url, gravatar


class GravatarTests(TestCase):
    def test_gravatar_url(self):
        self.assertEqual(
            "https://www.gravatar.com/avatar/738668e0913f0841e3285ca39fa2fc4b?s=80&d=",
            gravatar_url("admin@cjk.pl"),
        )
        self.assertEqual(
            "https://www.gravatar.com/avatar/738668e0913f0841e3285ca39fa2fc4b?s=120&d=",
            gravatar_url("admin@cjk.pl", 120),
        )

    def test_gavatar(self):
        html = '<img src="https://www.gravatar.com/avatar/738668e0913f0841e3285ca39fa2fc4b?s=80&d=" width="80" height="80"  >'  # noqa
        self.assertEqual(html, gravatar(email="admin@cjk.pl"))

        html = '<img src="https://www.gravatar.com/avatar/738668e0913f0841e3285ca39fa2fc4b?s=130&d=" width="130" height="130"  >'  # noqa
        self.assertEqual(html, gravatar(email="admin@cjk.pl", size=130))

        html = '<img src="https://www.gravatar.com/avatar/738668e0913f0841e3285ca39fa2fc4b?s=180&d=" width="180" height="180" class="avatar" >'  # noqa
        self.assertEqual(
            html, gravatar(email="admin@cjk.pl", size=180, options='class="avatar"')
        )


class GravatarWithDefaultGravatarUrlTests(TestCase):
    def test_gravatar_url(self):
        self.assertEqual(
            "https://www.gravatar.com/avatar/738668e0913f0841e3285ca39fa2fc4b?s=80&d=www.foo.com",
            gravatar_url(email="admin@cjk.pl", default_url="www.foo.com"),
        )
        self.assertEqual(
            "https://www.gravatar.com/avatar/738668e0913f0841e3285ca39fa2fc4b?s=120&d=www.foo.com",
            gravatar_url("admin@cjk.pl", 120, "www.foo.com"),
        )

    def test_gavatar(self):
        html = '<img src="https://www.gravatar.com/avatar/738668e0913f0841e3285ca39fa2fc4b?s=80&d=www.foo.com" width="80" height="80"  >'  # noqa
        self.assertEqual(
            html, gravatar(email="admin@cjk.pl", default_url="www.foo.com")
        )

        html = '<img src="https://www.gravatar.com/avatar/738668e0913f0841e3285ca39fa2fc4b?s=130&d=www.foo.com" width="130" height="130"  >'  # noqa
        self.assertEqual(html, gravatar("admin@cjk.pl", 130, "www.foo.com"))

        html = '<img src="https://www.gravatar.com/avatar/738668e0913f0841e3285ca39fa2fc4b?s=180&d=www.foo.com" width="180" height="180" class="avatar" >'  # noqa
        self.assertEqual(
            html, gravatar("admin@cjk.pl", 180, "www.foo.com", 'class="avatar"')
        )


# @TODO: add test to verify that an image added in wagtail backend/settings gets used as default



VERSION = (2022, 8, 7, "dev", 0)

__version_info__ = VERSION
__version__ = ".".join(map(str, VERSION[:3])) + (
    "-{}{}".format(VERSION[3], VERSION[4] or "") if VERSION[3] != "final" else ""
)
__author__ = "Grzegorz Król"
__license__ = "BSD-3-Clause"
__copyright__ = "Copyright 2021-2022 Grzegorz Król and contributors"



from .draftail_extensions import register_inline_styling, register_block_feature
from .draftail_icons import DRAFTAIL_ICONS

__all__ = ["register_inline_styling", "register_block_feature", "DRAFTAIL_ICONS"]



"""
Custom wagtail settings, defined per-site
"""

import json

from cjkcms.models.snippet_models import Footer, Navbar
from cjkcms.settings import cms_settings
from django.db import models
from django.utils.translation import gettext_lazy as _
from modelcluster.fields import ParentalKey
from modelcluster.models import ClusterableModel
from wagtail.admin.panels import FieldPanel, MultiFieldPanel, HelpPanel, InlinePanel
from wagtail.contrib.settings.models import BaseSiteSetting, register_setting
from wagtail.images import get_image_model_string
from wagtail.models import Orderable


@register_setting(icon="group")
class SocialMediaSettings(BaseSiteSetting):
    """
    Social media accounts.
    """

    class Meta:
        verbose_name = _("Social Media")
        app_label = "cjkcms"

    facebook = models.URLField(
        blank=True,
        verbose_name=_("Facebook"),
        help_text=_("Your Facebook page URL"),
    )
    twitter = models.URLField(
        blank=True,
        verbose_name=_("Twitter"),
        help_text=_("Your Twitter page URL"),
    )
    instagram = models.CharField(
        max_length=255,
        blank=True,
        verbose_name=_("Instagram"),
        help_text=_("Your Instagram username, without the @"),
    )
    linkedin = models.URLField(
        blank=True,
        verbose_name=_("LinkedIn"),
        help_text=_("Your LinkedIn page URL"),
    )
    youtube = models.URLField(
        blank=True,
        verbose_name=_("YouTube"),
        help_text=_("Your YouTube channel or user account URL"),
    )
    vimeo = models.URLField(
        blank=True,
        verbose_name=_("Vimeo"),
        help_text=_("Your Vimeo channel or user account URL"),
    )

    @property
    def twitter_handle(self):
        """
        Gets the handle of the twitter account from a URL.
        """
        return self.twitter.strip().strip("/").split("/")[-1]

    @property
    def social_json(self):
        """
        Returns non-blank social accounts as a JSON list.
        """
        socialist = [
            self.facebook,
            self.twitter,
            self.instagram,
            self.youtube,
            self.linkedin,
            self.vimeo,
        ]
        socialist = list(filter(None, socialist))
        return json.dumps(socialist)

    panels = [
        MultiFieldPanel(
            [
                FieldPanel("facebook"),
                FieldPanel("twitter"),
                FieldPanel("instagram"),
                FieldPanel("linkedin"),
                FieldPanel("youtube"),
                FieldPanel("vimeo"),
            ],
            _("Social Media Accounts"),
        )
    ]


@register_setting(icon="grip")
class LayoutSettings(ClusterableModel, BaseSiteSetting):
    """
    Branding, navbar, and theme settings.
    """

    class Meta:
        verbose_name = _("Layout")

    logo = models.ForeignKey(
        get_image_model_string(),
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name="+",
        verbose_name=_("Logo"),
        help_text=_("Brand logo used in the navbar and throughout the site"),
    )
    favicon = models.ForeignKey(
        get_image_model_string(),
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name="favicon",
        verbose_name=_("Favicon"),
    )
    navbar_color_scheme = models.CharField(
        blank=True,
        max_length=50,
        choices=None,
        default="",
        verbose_name=_("Navbar color scheme"),
        help_text=_(
            "Optimizes text and other navbar elements for use with light or dark backgrounds."
        ),  # noqa
    )
    navbar_class = models.CharField(
        blank=True,
        max_length=255,
        default="",
        verbose_name=_("Navbar CSS class"),
        help_text=_(
            'Custom classes applied to navbar e.g. "bg-light", "bg-dark", "bg-primary".'
        ),
    )
    navbar_fixed = models.BooleanField(
        default=False,
        verbose_name=_("Fixed navbar"),
        help_text=_("Fixed navbar will remain at the top of the page when scrolling."),
    )
    navbar_wrapper_fluid = models.BooleanField(
        default=True,
        verbose_name=_("Full width navbar"),
        help_text=_("The navbar will fill edge to edge."),
    )
    navbar_content_fluid = models.BooleanField(
        default=False,
        verbose_name=_("Full width navbar contents"),
        help_text=_("Content within the navbar will fill edge to edge."),
    )
    navbar_collapse_mode = models.CharField(
        blank=True,
        max_length=50,
        choices=None,
        default="",
        verbose_name=_("Collapse navbar menu"),
        help_text=_(
            "Control on what screen sizes to show and collapse the navbar menu links."
        ),
    )
    navbar_format = models.CharField(
        blank=True,
        max_length=50,
        choices=None,
        default="",
        verbose_name=_("Navbar format"),
    )
    navbar_search = models.BooleanField(
        default=True,
        verbose_name=_("Search box"),
        help_text=_("Show search box in navbar"),
    )

    navbar_langselector = models.BooleanField(
        default=True,
        verbose_name=_("Language selector"),
        help_text=_("Show lang choice dropdown in navbar"),
    )

    frontend_theme = models.CharField(
        blank=True,
        max_length=50,
        choices=None,
        default="",
        verbose_name=_("Theme variant"),
        help_text=cms_settings.CJKCMS_FRONTEND_THEME_HELP,
    )

    awesome_cdn = models.BooleanField(
        default=False,
        verbose_name=_("Font Awesome"),
        help_text=_("Load font awesome from CND"),
    )

    base_template = models.CharField(
        blank=True,
        max_length=200,
        default=cms_settings.CJKCMS_BASE_TEMPLATE_DEFAULT,
        verbose_name=_("Base template"),
        help_text=cms_settings.CJKCMS_BASE_TEMPLATE_HELP,
    )

    panels = [
        MultiFieldPanel(
            [
                FieldPanel("logo"),
                FieldPanel("favicon"),
            ],
            heading=_("Branding"),
        ),
        InlinePanel(
            "site_navbar",
            help_text=_("Choose one or more navbars"),
            heading=_("Site Navbars"),
        ),
        MultiFieldPanel(
            [
                FieldPanel("navbar_color_scheme"),
                FieldPanel("navbar_class"),
                FieldPanel("navbar_fixed"),
                FieldPanel("navbar_wrapper_fluid"),
                FieldPanel("navbar_content_fluid"),
                FieldPanel("navbar_collapse_mode"),
                FieldPanel("navbar_format"),
                FieldPanel("navbar_search"),
                FieldPanel("navbar_langselector"),
            ],
            heading=_("Site Navbar Layout"),
        ),
        InlinePanel(
            "site_footer",
            help_text=_("Choose one or more footers"),
            heading=_("Site Footers"),
        ),
        MultiFieldPanel(
            [
                FieldPanel("frontend_theme"),
                FieldPanel("base_template"),
                FieldPanel("awesome_cdn"),
            ],
            heading=_("Theming"),
        ),
    ]

    def __init__(self, *args, **kwargs):
        """
        Inject custom choices and defaults into the form fields
        to enable customization of settings without causing migration issues.
        """
        super().__init__(*args, **kwargs)
        # Set choices dynamically.
        self._meta.get_field(
            "frontend_theme"
        ).choices = cms_settings.CJKCMS_FRONTEND_THEME_CHOICES  # type: ignore
        self._meta.get_field(
            "navbar_collapse_mode"
        ).choices = cms_settings.CJKCMS_FRONTEND_NAVBAR_COLLAPSE_MODE_CHOICES  # type: ignore
        self._meta.get_field(
            "navbar_color_scheme"
        ).choices = cms_settings.CJKCMS_FRONTEND_NAVBAR_COLOR_SCHEME_CHOICES  # type: ignore
        self._meta.get_field(
            "navbar_format"
        ).choices = cms_settings.CJKCMS_FRONTEND_NAVBAR_FORMAT_CHOICES  # type: ignore
        # Set default dynamically.
        if not self.id:  # type: ignore # if new record / id not yet assigned
            self.frontend_theme = cms_settings.CJKCMS_FRONTEND_THEME_DEFAULT
            self.navbar_class = cms_settings.CJKCMS_FRONTEND_NAVBAR_CLASS_DEFAULT
            self.navbar_collapse_mode = (
                cms_settings.CJKCMS_FRONTEND_NAVBAR_COLLAPSE_MODE_DEFAULT
            )
            self.navbar_color_scheme = (
                cms_settings.CJKCMS_FRONTEND_NAVBAR_COLOR_SCHEME_DEFAULT
            )
            self.navbar_format = cms_settings.CJKCMS_FRONTEND_NAVBAR_FORMAT_DEFAULT
            self.base_template = cms_settings.CJKCMS_BASE_TEMPLATE_DEFAULT


class NavbarOrderable(Orderable, models.Model):
    navbar_chooser = ParentalKey(
        LayoutSettings, related_name="site_navbar", verbose_name=_("Site Navbars")
    )
    navbar = models.ForeignKey(
        Navbar,
        blank=True,
        null=True,
        on_delete=models.CASCADE,
    )

    panels = [FieldPanel("navbar")]


class FooterOrderable(Orderable, models.Model):
    footer_chooser = ParentalKey(
        LayoutSettings, related_name="site_footer", verbose_name=_("Site Footers")
    )
    footer = models.ForeignKey(
        Footer,
        blank=True,
        null=True,
        on_delete=models.CASCADE,
    )

    panels = [FieldPanel("footer")]


@register_setting(icon="history")
class AnalyticsSettings(BaseSiteSetting):
    """
    Tracking and Google Analytics.
    """

    class Meta:
        verbose_name = _("Tracking")

    ga_tracking_id = models.CharField(
        blank=True,
        max_length=255,
        verbose_name=_("Universal Analytics Tracking ID"),
        help_text=_(
            'Google Analytics tracking ID (begins with "UA-"). '
            "Note: will stop working by 1 July 2023!"
        ),
    )
    g4_tracking_id = models.CharField(
        blank=True,
        max_length=255,
        verbose_name=_("G4 Analytics Tracking ID"),
        help_text=_(
            'G4 Google Analytics tracking ID ("G-..."). If both G4 & UA set, both will be used.'
        ),
    )
    ga_track_button_clicks = models.BooleanField(
        default=False,
        verbose_name=_("Track button clicks"),
        help_text=_(
            "Track all button clicks using Google Analytics event tracking. "
            "Event tracking details can be specified in each button’s advanced settings options."
        ),  # noqa
    )
    gtm_id = models.CharField(
        blank=True,
        max_length=255,
        verbose_name=_("Google Tag Manager ID"),
        help_text=_('Begins with "GTM-"'),
    )

    panels = [
        HelpPanel(
            heading=_("Know your tracking"),
            content=_(
                "<h3><b>Which tracking IDs do I need?</b></h3>"
                "<p>Before adding tracking to your site, "
                '<a href="https://docs.coderedcorp.com/wagtail-crx/how_to/add_tracking_scripts.html" '  # noqa
                'target="_blank">read about the difference between UA, G, GTM, '
                "and other tracking IDs</a>.</p>"
            ),
        ),
        MultiFieldPanel(
            [
                FieldPanel("ga_tracking_id"),
                FieldPanel("g4_tracking_id"),
                FieldPanel("ga_track_button_clicks"),
            ],
            heading=_("Google Analytics"),
        ),
        MultiFieldPanel(
            [
                FieldPanel("gtm_id"),
            ],
            heading=_("Google Tag Manager"),
        ),
    ]


@register_setting(icon="cog")
class GeneralSettings(BaseSiteSetting):
    """
    Various site-wide settings. A good place to put
    one-off settings that don't belong anywhere else.
    """

    from_email_address = models.CharField(
        blank=True,
        max_length=255,
        verbose_name=_("From email address"),
        help_text=_(
            "The default email address this site appears to send from. "
            'For example: "sender@example.com" or '
            '"Sender Name <sender@example.com>" (without quotes)'
        ),
        # noqa
    )
    search_num_results = models.PositiveIntegerField(
        default=20,
        verbose_name=_("Number of results per page"),
    )
    external_new_tab = models.BooleanField(
        default=False, verbose_name=_("Open all external links in new tab")
    )

    gravatar = models.ForeignKey(
        get_image_model_string(),
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name="+",
        verbose_name=_("Gravatar Default"),
        help_text=_("Gravatar image when user email not found in gravatar db"),
    )

    panels = [
        MultiFieldPanel(
            [
                FieldPanel("from_email_address"),
            ],
            _("Email"),
        ),
        MultiFieldPanel(
            [
                FieldPanel("search_num_results"),
            ],
            _("Search Settings"),
        ),
        MultiFieldPanel(
            [
                FieldPanel("external_new_tab"),
            ],
            _("Links"),
        ),
        MultiFieldPanel(
            [
                FieldPanel("gravatar"),
            ],
            _("Avatar"),
        ),
    ]

    class Meta:
        verbose_name = _("General")


@register_setting(icon="mail")
class MailchimpApiSettings(BaseSiteSetting):
    """
    Settings for Mailchimp API services.
    """

    class Meta:
        verbose_name = _("Mailchimp API")

    mailchimp_api_key = models.CharField(
        blank=True,
        max_length=255,
        verbose_name=_("Mailchimp API Key"),
        help_text=_("The API Key used for Mailchimp."),
    )


@register_setting(icon="password")
class AdobeApiSettings(BaseSiteSetting):
    """
    Settings for Adobe PDF Embed services
    """

    adobe_embed_id = models.CharField(
        null=True,
        blank=True,
        max_length=64,
        verbose_name=_("Adobe Embed API ID"),
        help_text=_("Adobe PDF Embed API Client ID"),
    )



import pytest

from wagtail.core.models import Page
from django.urls import reverse
from django.test import Client, TestCase

from cjkcms.models.cms_models import ArticlePage


@pytest.mark.django_db
class TestSearchBlocks(TestCase):
    """
    CjkCMS content blocks should be searchable.
    For each content block, create a page with that content block and some text,
    confirm that the search returns the page.
    """

    block_richtext = [
        {
            "type": "text",
            "value": '<p data-block-key="o94f7"> When daisies pied and violets blue</p>',
            "id": "bbfc2acb-79b3-47a6-b974-2a6e4523c94b",
        }
    ]

    block_button_link = [
        {
            "type": "button",
            "value": {
                "settings": {},
                "page_link": None,
                "doc_link": None,
                "other_link": "http://youtube.com",
                "button_title": "My buttony button, not Benjamin",
                "button_style": "btn-primary",
                "button_size": "",
                "visible_for": "all",
            },
            "id": "3f02e0d5-5b99-4e43-821d-549af74b3014",
        }
    ]

    block_html = [
        {
            "type": "html",
            "value": "<h1>Hi from HTML</h1>",
            "id": "d07fc66f-c94c-4ee8-a113-0c17ed2220e5",
        }
    ]

    block_quote = [
        {
            "type": "quote",
            "value": {
                "settings": {
                    "custom_template": "",
                    "custom_css_class": "",
                    "custom_id": "",
                },
                "text": "Quotably interesting",
                "author": "By Mr Nobody",
            },
            "id": "a586b8eb-3af4-4e8a-9f12-c6ec582930bf",
        }
    ]

    def setUp(self):
        self.client = Client()
        self.create_article_page()

    def create_article_page(self):
        # Get the HomePage
        home_page = Page.objects.get(path="00010001")

        article_page = ArticlePage(title="Test Article", body=None)

        home_page.add_child(instance=article_page)
        article_page.save_revision().publish()

    def set_article_body(self, content) -> None:
        article_page = ArticlePage.objects.get(title="Test Article")
        article_page.body = content
        article_page.save_revision().publish()

    def test_page_type_in_search_results(self):
        response = self.client.get(
            reverse("cjkcms_search"), {"s": "doesn't matter"}, follow=True
        )

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.context["pagetypes"][0].__name__, "ArticlePage")

    def test_search_richtext(self):
        self.set_article_body(self.block_richtext)

        response = self.client.get(
            reverse("cjkcms_search"), {"s": "not-there"}, follow=True
        )

        self.assertEqual(response.context["results"].count(), 0)

        response = self.client.get(
            reverse("cjkcms_search"), {"s": "daisies"}, follow=True
        )

        self.assertEqual(response.context["results"].count(), 1)

    def test_search_button(self):
        self.set_article_body(self.block_button_link)

        response = self.client.get(
            reverse("cjkcms_search"), {"s": "daisies"}, follow=True
        )

        self.assertEqual(response.context["results"].count(), 0)

        response = self.client.get(
            reverse("cjkcms_search"), {"s": "Benjamin"}, follow=True
        )

        self.assertEqual(response.context["results"].count(), 1)

    def test_search_html(self):
        self.set_article_body(self.block_html)

        response = self.client.get(
            reverse("cjkcms_search"), {"s": "can't see me"}, follow=True
        )

        self.assertEqual(response.context["results"].count(), 0)

        response = self.client.get(
            reverse("cjkcms_search"), {"s": "from HTML"}, follow=True
        )

        self.assertEqual(response.context["results"].count(), 1)

    def test_search_quote(self):
        self.set_article_body(self.block_quote)

        response = self.client.get(
            reverse("cjkcms_search"), {"s": "from HTML"}, follow=True
        )

        self.assertEqual(response.context["results"].count(), 0)

        response = self.client.get(
            reverse("cjkcms_search"), {"s": "quotably"}, follow=True
        )

        self.assertEqual(response.context["results"].count(), 1)

        response = self.client.get(
            reverse("cjkcms_search"), {"s": "Nobody"}, follow=True
        )

        self.assertEqual(response.context["results"].count(), 1)



"""
Snippets are for content that is re-usable in nature.
"""

from django.db import models
from django.utils.text import slugify
from django.utils.translation import gettext_lazy as _
from modelcluster.fields import ParentalKey
from modelcluster.models import ClusterableModel
from wagtail.admin.panels import FieldPanel, InlinePanel, MultiFieldPanel
from wagtail.core.models import (
    Orderable,
)  # problems with unique slug! <- TranslatableMixin
from wagtail.snippets.models import register_snippet
from wagtail.images import get_image_model_string

from cjkcms.blocks import (
    HTML_STREAMBLOCKS,
    LAYOUT_STREAMBLOCKS,
    NAVIGATION_STREAMBLOCKS,
)
from cjkcms.settings import cms_settings
from cjkcms.fields import CjkcmsStreamField


@register_snippet
class Carousel(ClusterableModel):
    """
    Model that represents a Carousel. Can be modified through the snippets UI.
    Selected through Page StreamField bodies by the CarouselSnippetChooser in
    snippet_choosers.py
    """

    class Meta:
        verbose_name = _("Carousel")

    name = models.CharField(
        max_length=255,
        verbose_name=_("Name"),
    )
    show_controls = models.BooleanField(
        default=True,
        verbose_name=_("Show controls"),
        help_text=_(
            "Shows arrows on the left and right of the carousel to advance next or previous slides."
        ),  # noqa
    )
    show_indicators = models.BooleanField(
        default=True,
        verbose_name=_("Show indicators"),
        help_text=_(
            "Shows small indicators at the bottom of the carousel based on the number of slides."
        ),  # noqa
    )
    animation = models.CharField(
        blank=True,
        max_length=20,
        choices=None,
        default="",
        verbose_name=_("Animation"),
        help_text=_("The animation when transitioning between slides."),
    )

    panels = [
        MultiFieldPanel(
            heading=_("Slider"),
            children=[
                FieldPanel("name"),
                FieldPanel("show_controls"),
                FieldPanel("show_indicators"),
                FieldPanel("animation"),
            ],
        ),
        InlinePanel("carousel_slides", label=_("Slides")),
    ]

    def __str__(self):
        return self.name

    def __init__(self, *args, **kwargs):
        """
        Inject custom choices and defaults into the form fields
        to enable customization of settings without causing migration issues.
        """
        super().__init__(*args, **kwargs)
        # Set choices dynamically.
        self._meta.get_field(
            "animation"
        ).choices = cms_settings.CJKCMS_FRONTEND_CAROUSEL_FX_CHOICES  # type: ignore
        # Set default dynamically.
        if not self.id:  # type: ignore
            self.animation = cms_settings.CJKCMS_FRONTEND_CAROUSEL_FX_DEFAULT


class CarouselSlide(Orderable, models.Model):
    """
    Represents a slide for the Carousel model. Can be modified through the
    snippets UI.
    """

    class Meta:
        verbose_name = _("Carousel Slide")

    carousel = ParentalKey(
        Carousel,
        related_name="carousel_slides",
        verbose_name=_("Carousel"),
    )
    image = models.ForeignKey(
        get_image_model_string(),
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name="+",
        verbose_name=_("Image"),
    )
    background_color = models.CharField(
        max_length=255,
        blank=True,
        verbose_name=_("Background color"),
        help_text=_("Hexadecimal, rgba, or CSS color notation (e.g. #ff0011)"),
    )
    custom_css_class = models.CharField(
        max_length=255,
        blank=True,
        verbose_name=_("Custom CSS class"),
    )
    custom_id = models.CharField(
        max_length=255,
        blank=True,
        verbose_name=_("Custom ID"),
    )
    content = CjkcmsStreamField(HTML_STREAMBLOCKS, blank=True, use_json_field=True)

    panels = [
        FieldPanel("image"),
        FieldPanel("background_color"),
        FieldPanel("custom_css_class"),
        FieldPanel("custom_id"),
        FieldPanel("content"),
    ]


@register_snippet
class Classifier(ClusterableModel):
    """
    Simple and generic model to organize/categorize/group pages.
    """

    class Meta:
        verbose_name = _("Classifier")
        verbose_name_plural = _("Classifiers")
        ordering = ["name"]

    slug = models.SlugField(
        allow_unicode=True,
        unique=True,
        verbose_name=_("Slug"),
    )
    name = models.CharField(
        max_length=255,
        verbose_name=_("Name"),
    )

    panels = [FieldPanel("name"), InlinePanel("terms", label=_("Classifier Terms"))]

    def save(self, *args, **kwargs):
        if not self.slug:
            # Make a slug and suffix a number if it already exists to ensure uniqueness
            newslug = slugify(self.name, allow_unicode=True)
            tmpslug = newslug
            suffix = 1
            while True:
                if not Classifier.objects.filter(slug=tmpslug).exists():
                    self.slug = tmpslug
                    break
                tmpslug = f"{newslug}-{str(suffix)}"
                suffix += 1
        return super().save(*args, **kwargs)

    def __str__(self):
        return self.name


class ClassifierTerm(Orderable, models.Model):
    """
    Term used to categorize a page.
    """

    class Meta:
        verbose_name = _("Classifier Term")
        verbose_name_plural = _("Classifier Terms")

    classifier = ParentalKey(
        Classifier,
        related_name="terms",
        verbose_name=_("Classifier"),
    )
    slug = models.SlugField(
        allow_unicode=True,
        unique=True,
        verbose_name=_("Slug"),
    )
    name = models.CharField(
        max_length=255,
        verbose_name=_("Name"),
    )

    panels = [
        FieldPanel("name"),
    ]

    def save(self, *args, **kwargs):
        if not self.slug:
            # Make a slug and suffix a number if it already exists to ensure uniqueness
            newslug = slugify(self.name, allow_unicode=True)
            tmpslug = newslug
            suffix = 1
            while True:
                if not ClassifierTerm.objects.filter(slug=tmpslug).exists():
                    self.slug = tmpslug
                    break
                tmpslug = f"{newslug}-{str(suffix)}"
                suffix += 1
        return super().save(*args, **kwargs)

    def __str__(self):
        return "{0} > {1}".format(self.classifier.name, self.name)


@register_snippet
class Navbar(models.Model):
    """
    Snippet for site navigation bars (header, main menu, etc.)
    """

    class Meta:
        verbose_name = _("Navigation Bar")

    name = models.CharField(
        max_length=255,
        verbose_name=_("Name"),
    )
    custom_css_class = models.CharField(
        max_length=255,
        blank=True,
        verbose_name=_("Custom CSS Class"),
    )
    custom_id = models.CharField(
        max_length=255,
        blank=True,
        verbose_name=_("Custom ID"),
    )
    menu_items = CjkcmsStreamField(
        NAVIGATION_STREAMBLOCKS,
        verbose_name=_("Navigation links"),
        use_json_field=True,
    )

    panels = [
        FieldPanel("name"),
        MultiFieldPanel(
            [
                FieldPanel("custom_css_class"),
                FieldPanel("custom_id"),
            ],
            heading=_("Attributes"),
        ),
        FieldPanel("menu_items"),
    ]

    def __str__(self):
        return self.name


@register_snippet
class Footer(models.Model):
    """
    Snippet for website footer content.
    """

    class Meta:
        verbose_name = _("Footer")

    name = models.CharField(
        max_length=255,
        verbose_name=_("Name"),
    )
    custom_css_class = models.CharField(
        max_length=255,
        blank=True,
        verbose_name=_("Custom CSS Class"),
    )
    custom_id = models.CharField(
        max_length=255,
        blank=True,
        verbose_name=_("Custom ID"),
    )
    content = CjkcmsStreamField(
        LAYOUT_STREAMBLOCKS, verbose_name=_("Content"), use_json_field=True
    )

    panels = [
        FieldPanel("name"),
        MultiFieldPanel(
            [
                FieldPanel("custom_css_class"),
                FieldPanel("custom_id"),
            ],
            heading=_("Attributes"),
        ),
        FieldPanel("content"),
    ]

    def __str__(self):
        return self.name


@register_snippet
class ReusableContent(models.Model):
    """
    Snippet for resusable content in streamfields.
    """

    class Meta:
        verbose_name = _("Reusable Content")
        verbose_name_plural = _("Reusable Content")

    name = models.CharField(
        max_length=255,
        verbose_name=_("Name"),
    )
    content = CjkcmsStreamField(
        LAYOUT_STREAMBLOCKS, verbose_name=_("content"), use_json_field=True
    )

    panels = [FieldPanel("name"), FieldPanel("content")]

    def __str__(self):
        return self.name


@register_snippet
class Accordion(ClusterableModel):
    """Class for reusable content in a collapsible block."""

    class Meta:
        verbose_name = _("Accordion")
        verbose_name_plural = _("Accordions")

    name = models.CharField(
        max_length=255,
        verbose_name=_("Name"),
    )

    panels = [
        MultiFieldPanel(
            heading=_("Accordion"),
            children=[
                FieldPanel("name"),
            ],
        ),
        InlinePanel("accordion_panels", label=_("Panels")),
    ]

    def __str__(self):
        return self.name


class AccordionPanel(Orderable, models.Model):
    """A panel for a collapsible accordion"""

    accordion = ParentalKey(
        Accordion,
        related_name="accordion_panels",
        verbose_name=_("Accordion"),
    )

    name = models.CharField(
        max_length=255,
        verbose_name=_("Name"),
    )

    content = CjkcmsStreamField(
        HTML_STREAMBLOCKS,
        blank=True,
        use_json_field=True,
    )

    custom_css_class = models.CharField(
        max_length=255,
        blank=True,
        verbose_name=_("Custom CSS class"),
    )
    custom_id = models.CharField(
        max_length=255,
        blank=True,
        verbose_name=_("Custom ID"),
    )

    panels = [
        FieldPanel("custom_css_class"),
        FieldPanel("custom_id"),
        FieldPanel("name"),
        FieldPanel("content"),
    ]


class CjkcmsEmail(ClusterableModel):
    """
    General purpose abstract clusterable model used for holding email information.
    Most likely this should be subclassed with addition of a ParentalKey.
    """

    class Meta:
        abstract = True
        verbose_name = _("CMS Email")

    name = models.CharField(
        max_length=255,
        blank=True,
        verbose_name=_("Template Name"),
    )

    to_address = models.CharField(
        max_length=255,
        blank=True,
        verbose_name=_("To Addresses"),
        help_text=_("Separate multiple email addresses with commas."),
    )
    from_address = models.CharField(
        max_length=255,
        blank=True,
        verbose_name=_("From Address"),
        help_text=_(
            'For example: "sender@abc.com" or "Sender Name <sender@abc.com>" (without quotes).'
        ),  # noqa
    )
    reply_address = models.CharField(
        max_length=255,
        blank=True,
        verbose_name=_("Reply-To Address"),
        help_text=_("Separate multiple email addresses with commas."),
    )
    cc_address = models.CharField(
        max_length=255,
        blank=True,
        verbose_name=_("CC"),
        help_text=_("Separate multiple email addresses with commas."),
    )
    bcc_address = models.CharField(
        max_length=255,
        blank=True,
        verbose_name=_("BCC"),
        help_text=_("Separate multiple email addresses with commas."),
    )
    subject = models.CharField(max_length=255, blank=True, verbose_name=_("Subject"))
    body = models.TextField(blank=True, verbose_name=_("Body"))

    panels = [
        MultiFieldPanel(
            [
                FieldPanel("to_address"),
                FieldPanel("from_address"),
                FieldPanel("cc_address"),
                FieldPanel("bcc_address"),
                FieldPanel("subject"),
                FieldPanel("body"),
            ],
            _("Email Message"),
        ),
    ]

    def __str__(self):
        return self.subject



from django.conf import settings


class _DefaultSettings:

    CJKCMS_BRAND_LOGO_LONG = "cjkcms/images/logos/cms-logo-long.svg"
    CJKCMS_BRAND_LOGO_SQUARE = "cjkcms/images/logos/cms-logo-square.svg"
    CJKCMS_FRONTEND_BTN_SIZE_DEFAULT = ""
    CJKCMS_FRONTEND_BTN_SIZE_CHOICES = [
        ("btn-sm", "Small"),
        ("", "Default"),
        ("btn-lg", "Large"),
    ]

    CJKCMS_FRONTEND_BTN_STYLE_DEFAULT = "btn-primary"
    CJKCMS_FRONTEND_BTN_STYLE_CHOICES = [
        ("btn-primary", "Primary"),
        ("btn-secondary", "Secondary"),
        ("btn-success", "Success"),
        ("btn-danger", "Danger"),
        ("btn-warning", "Warning"),
        ("btn-info", "Info"),
        ("btn-link", "Link"),
        ("btn-light", "Light"),
        ("btn-dark", "Dark"),
        ("btn-outline-primary", "Outline Primary"),
        ("btn-outline-secondary", "Outline Secondary"),
        ("btn-outline-success", "Outline Success"),
        ("btn-outline-danger", "Outline Danger"),
        ("btn-outline-warning", "Outline Warning"),
        ("btn-outline-info", "Outline Info"),
        ("btn-outline-light", "Outline Light"),
        ("btn-outline-dark", "Outline Dark"),
    ]

    CJKCMS_FRONTEND_CAROUSEL_FX_DEFAULT = ""
    CJKCMS_FRONTEND_CAROUSEL_FX_CHOICES = [
        ("", "Slide"),
        ("carousel-fade", "Fade"),
    ]

    CJKCMS_FRONTEND_COL_SIZE_DEFAULT = ""
    CJKCMS_FRONTEND_COL_SIZE_CHOICES = [
        ("", "Automatically size"),
        ("12", "Full row"),
        ("6", "Half - 1/2 column"),
        ("4", "Thirds - 1/3 column"),
        ("8", "Thirds - 2/3 column"),
        ("3", "Quarters - 1/4 column"),
        ("9", "Quarters - 3/4 column"),
        ("2", "Sixths - 1/6 column"),
        ("10", "Sixths - 5/6 column"),
        ("1", "Twelfths - 1/12 column"),
        ("5", "Twelfths - 5/12 column"),
        ("7", "Twelfths - 7/12 column"),
        ("11", "Twelfths - 11/12 column"),
    ]

    CJKCMS_FRONTEND_COL_BREAK_DEFAULT = "md"
    CJKCMS_FRONTEND_COL_BREAK_CHOICES = [
        ("", "Always expanded"),
        ("sm", "sm - Expand on small screens (phone, 576px) and larger"),
        ("md", "md - Expand on medium screens (tablet, 768px) and larger"),
        ("lg", "lg - Expand on large screens (laptop, 992px) and larger"),
        ("xl", "xl - Expand on extra large screens (wide monitor, 1200px)"),
    ]

    CJKCMS_FRONTEND_NAVBAR_FORMAT_DEFAULT = ""
    CJKCMS_FRONTEND_NAVBAR_FORMAT_CHOICES = [
        ("", "Default Bootstrap Navbar"),
        ("cjkcms-navbar-center", "Centered logo at top"),
    ]

    CJKCMS_FRONTEND_NAVBAR_COLOR_SCHEME_DEFAULT = "navbar-light"
    CJKCMS_FRONTEND_NAVBAR_COLOR_SCHEME_CHOICES = [
        ("navbar-light", "Light - for use with a light-colored navbar"),
        ("navbar-dark", "Dark - for use with a dark-colored navbar"),
    ]

    CJKCMS_FRONTEND_NAVBAR_CLASS_DEFAULT = "bg-light"

    CJKCMS_FRONTEND_NAVBAR_COLLAPSE_MODE_DEFAULT = "navbar-expand-lg"
    CJKCMS_FRONTEND_NAVBAR_COLLAPSE_MODE_CHOICES = [
        ("", "Never show menu - Always collapse menu behind a button"),
        ("navbar-expand-sm", "sm - Show on small screens (phone size) and larger"),
        ("navbar-expand-md", "md - Show on medium screens (tablet size) and larger"),
        ("navbar-expand-lg", "lg - Show on large screens (laptop size) and larger"),
        (
            "navbar-expand-xl",
            "xl - Show on extra large screens (desktop, wide monitor)",
        ),
    ]

    CJKCMS_FRONTENT_NAVBAR_SEARCHBOX_CLASS = "border-secondary mb-0"
    CJKCMS_FRONTEND_THEME_HELP = "Change the source of your Bootstrap theme."
    CJKCMS_FRONTEND_THEME_DEFAULT = ""
    CJKCMS_FRONTEND_THEME_CHOICES = (
        ("", "Default - Built-in Bootstrap 5"),
        ("mdb.light", "Built-in MDBootstrap 5"),
        ("mdb.dark", "Built-in MDBootstrap 5 dark"),
        ("python-webpack", "Python Webpack Boilerplate by Michael Yin"),
    )

    CJKCMS_FRONTEND_TEMPLATES_BLOCKS = {
        "cardblock": [
            ("cjkcms/blocks/card_block.html", "Card"),
            ("cjkcms/blocks/card_horizontal.html", "Horizontal Card"),
            ("cjkcms/blocks/card_head.html", "Card with header"),
            ("cjkcms/blocks/card_foot.html", "Card with footer"),
            ("cjkcms/blocks/card_head_foot.html", "Card with header and footer"),
            ("cjkcms/blocks/card_blurb.html", "Blurb - rounded image and no border"),
            ("cjkcms/blocks/card_img.html", "Cover image - use image as background"),
            ("cjkcms/blocks/card_landing1.html", "Landing page style 1"),
            ("cjkcms/blocks/card_landing2.html", "Landing page style 2"),
        ],
        "cardgridblock": [
            (
                "cjkcms/blocks/cardgrid_group.html",
                "Card group - attached cards of equal size",
            ),
            (
                "cjkcms/blocks/cardgrid_deck.html",
                "Card deck - separate cards of equal size",
            ),
            (
                "cjkcms/blocks/cardgrid_columns.html",
                "Card masonry - fluid brick pattern",
            ),
        ],
        "pagelistblock": [
            ("cjkcms/blocks/pagelist_block.html", "General, simple list"),
            (
                "cjkcms/blocks/pagelist_list_group.html",
                "General, list group navigation panel",
            ),
            ("cjkcms/blocks/pagelist_article_media.html", "Article, media format"),
            (
                "cjkcms/blocks/pagelist_article_card_group.html",
                "Article, card group - attached cards of equal size",
            ),
            (
                "cjkcms/blocks/pagelist_article_card_deck.html",
                "Article, card deck - separate cards of equal size",
            ),
            (
                "cjkcms/blocks/pagelist_article_card_columns.html",
                "Article, card masonry - fluid brick pattern",
            ),
        ],
        "pagepreviewblock": [
            ("cjkcms/blocks/pagepreview_card.html", "Card"),
        ],
        # templates that are available for all block types
        "*": [
            ("", "Default"),
        ],
    }

    CJKCMS_FRONTEND_TEMPLATES_PAGES = {
        # templates that are available for all page types
        "*": [
            ("", "Default"),
            ("cjkcms/pages/web_page.html", "Web page showing title and cover image"),
            (
                "cjkcms/pages/web_page_notitle.html",
                "Web page without title and cover image",
            ),
            ("cjkcms/pages/base.html", "Blank page - no navbar or footer"),
        ],
    }

    CJKCMS_BANNER = None
    CJKCMS_BANNER_BACKGROUND = "#f00"
    CJKCMS_BANNER_TEXT_COLOR = "#fff"

    CJKCMS_BASE_TEMPLATE_HELP = (
        "Base template used by CMS pages, defaults to CMS built-in one"
    )
    CJKCMS_BASE_TEMPLATE_DEFAULT = "cjkcms/pages/base.html"
    CJKCMS_AUTH_VISIBILITY_DEFAULT = "all"
    CJKCMS_AUTH_VISIBILITY_CHOICES = (
        ("all", "Default (Everyone)"),
        ("non-auth-only", "Not logged in only"),
        ("auth-only", "Logged in only"),
        ("hidden", "Hidden for all"),
    )

    BANNER = None
    BANNER_BACKGROUND = "#f00"
    BANNER_TEXT_COLOR = "#fff"

    CJKCMS_RICHTEXT_FEATURES = {
        "default": [
            "h2",
            "h3",
            "h4",
            "bold",
            "italic",
            "link",
            "ol",
            "ul",
            "hr",
            "blockquote",
            "image",
            "centre-align",
            "left-align",
            "right-align",
        ],
        "full": [
            "h2",
            "h3",
            "h4",
            "h5",
            "h6",
            "bold",
            "italic",
            "underline",
            "ol",
            "ul",
            "larger",
            "smaller",
            "superscript",
            "subscript",
            "strikethrough",
            "link",
            "hr",
            "code",
            "document-link",
            "blockquote",
            "image",
            "embed",
            "centre-align",
            "left-align",
            "right-align",
        ],
        "minimal": ["bold", "italic", "link"],
    }

    def __getattribute__(self, attr: str):
        # First load from Django settings.
        # If it does not exist, load from _DefaultSettings.
        try:
            return getattr(settings, attr)
        except AttributeError:
            return super().__getattribute__(attr)


cms_settings = _DefaultSettings()



from wagtail.admin.rich_text.converters.html_to_contentstate import (
    BlockElementHandler,
    InlineStyleElementHandler,
)
from wagtail.admin.rich_text.editors.draftail.features import InlineStyleFeature

import wagtail.admin.rich_text.editors.draftail.features as draftail_features


def register_inline_styling(
    features,
    feature_name,
    description,
    type_,
    tag="span",
    format=None,
    editor_style=None,
    label=None,
    icon=None,
):
    control = {"type": type_, "description": description}
    if label:
        control["label"] = label
    elif icon:
        control["icon"] = icon
    else:
        control["label"] = description
    if editor_style:
        control["style"] = editor_style

    if not format:
        style_map = {"element": tag}
        markup_map = tag
    else:
        style_map = f"{tag} {format}"
        markup_map = f"{tag}[{format}]"

    features.register_editor_plugin(
        "draftail", feature_name, InlineStyleFeature(control)
    )
    db_conversion = {
        "from_database_format": {markup_map: InlineStyleElementHandler(type_)},
        "to_database_format": {"style_map": {type_: style_map}},
    }
    features.register_converter_rule("contentstate", feature_name, db_conversion)


def register_block_feature(
    features,
    feature_name,
    description,
    type_,
    css_class,
    element="div",
    label=None,
    icon=None,
    editor_style=None,
):
    control = {
        "type": type_,
        "description": description,
        "element": element,
    }
    if label:
        control["label"] = label
    elif icon:
        control["icon"] = icon
    else:
        control["label"] = description
    if editor_style:
        control["style"] = editor_style

    features.register_editor_plugin(
        "draftail",
        feature_name,
        draftail_features.BlockFeature(control),
    )

    features.register_converter_rule(
        "contentstate",
        feature_name,
        {
            "from_database_format": {
                f"{element}[class={css_class}]": BlockElementHandler(type_)
            },
            "to_database_format": {
                "block_map": {
                    type_: {"element": element, "props": {"class": css_class}}
                }
            },
        },
    )



import pytest
import unittest

from django.urls import reverse
from django.test import Client
from django.test.utils import override_settings

from wagtail.core.models import Site
from wagtail.images.tests.utils import Image, get_test_image_file

from cjkcms.models import LayoutSettings

# from cjkcms.tests.testapp.models import EventPage, EventIndexPage, EventOccurrence


@pytest.mark.django_db
class TestSiteURLs(unittest.TestCase):
    def setUp(self):
        self.client = Client()

    # @TODO: this fails, even though in browser tests returns 404
    @override_settings(DEBUG=False)
    def test_404(self):
        response = self.client.get("/testing/404/page/")
        self.assertEqual(response.status_code, 404)

    def test_sitemap(self):
        response = self.client.get("/sitemap.xml")

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response["content-type"], "application/xml")

    def test_robots(self):
        response = self.client.get("/robots.txt")

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response["content-type"], "text/plain")

    def test_search(self):
        response = self.client.get(
            reverse("cjkcms_search"), {"s": "Test Search Query"}, follow=True
        )

        self.assertEqual(response.status_code, 200)
        self.assertNotEqual(response.context["results"], None)

        response = self.client.get(
            reverse("cjkcms_search"),
            {
                "s": "keyword",
                "t": "t",
            },
            follow=False,
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.context["results"], None)


@pytest.mark.django_db
class TestFavicon(unittest.TestCase):
    def test_404(self):
        client = Client()
        # Get the default site
        site = Site.objects.filter(is_default_site=True)[0]
        # Ensure the favicon is blank
        layout = LayoutSettings.for_site(site)
        layout.favicon = None
        layout.save()
        # Expect a 404
        response = client.get("/favicon.ico")
        self.assertEqual(response.status_code, 404)

    def test_301(self):
        client = Client()
        # Get the default site
        site = Site.objects.filter(is_default_site=True)[0]
        # Set a dummy favicon
        layout = LayoutSettings.for_site(site)
        img = Image.objects.create(
            title="Test image",
            file=get_test_image_file(),
        )
        layout.favicon = img
        layout.save()
        # Expect a 301 redirect
        response = client.get("/favicon.ico")
        self.assertEqual(response.status_code, 301)



"""
Blocks for StreamField content
"""

from django import forms
from django.template.loader import render_to_string
from django.utils.functional import cached_property
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _
from wagtail.core import blocks
from wagtail.core.models import Collection
from wagtail.core.utils import resolve_model_string
from wagtail.documents.blocks import DocumentChooserBlock

from cjkcms.settings import cms_settings


class ClassifierTermChooserBlock(blocks.FieldBlock):
    """
    Enables choosing a ClassifierTerm in the streamfield.
    Lazy loads the target_model from the string to avoid recursive imports.
    """

    widget = forms.Select
    _help_text = _("Choose a Classifier Term")

    def __init__(self, required=False, label=None, help_text=None, *args, **kwargs):
        self._required = required
        if help_text:
            self._help_text = help_text
        self._label = label
        super().__init__(*args, **kwargs)

    @cached_property
    def target_model(self):
        return resolve_model_string("cjkcms.ClassifierTerm")

    @cached_property
    def field(self):
        return forms.ModelChoiceField(
            queryset=self.target_model.objects.all().order_by(  # type: ignore
                "classifier__name", "name"
            ),
            widget=self.widget,
            required=self._required,
            label=self._label,
            help_text=self._help_text,
        )

    def to_python(self, value):
        """
        Convert the serialized value back into a python object.
        """
        if isinstance(value, int):
            return self.target_model.objects.get(pk=value)  # type: ignore
        return value

    def get_prep_value(self, value):
        """
        Serialize the model in a form suitable for wagtail's JSON-ish streamfield
        """
        return value.pk if isinstance(value, self.target_model) else value


class CollectionChooserBlock(blocks.FieldBlock):
    """
    Enables choosing a wagtail Collection in the streamfield.
    """

    target_model = Collection
    widget = forms.Select
    _help_text = _("Choose a collection")

    def __init__(self, required=False, label=None, help_text=None, *args, **kwargs):
        self._required = required
        if help_text:
            self._help_text = help_text
        self._label = label
        super().__init__(*args, **kwargs)

    @cached_property
    def field(self):
        return forms.ModelChoiceField(
            queryset=self.target_model.objects.all().order_by("name"),
            widget=self.widget,
            required=self._required,
            label=self._label,
            help_text=self._help_text,
        )

    def to_python(self, value):
        """
        Convert the serialized value back into a python object.
        """
        if isinstance(value, int):
            return self.target_model.objects.get(pk=value)
        return value

    def get_prep_value(self, value):
        """
        Serialize the model in a form suitable for wagtail's JSON-ish streamfield
        """
        return value.pk if isinstance(value, self.target_model) else value


class ButtonMixin(blocks.StructBlock):
    """
    Standard style and size options for buttons.
    """

    button_title = blocks.CharBlock(
        max_length=255,
        required=True,
        label=_("Button Title"),
    )
    button_style = blocks.ChoiceBlock(
        choices=cms_settings.CJKCMS_FRONTEND_BTN_STYLE_CHOICES,
        default=cms_settings.CJKCMS_FRONTEND_BTN_STYLE_DEFAULT,
        required=False,
        label=_("Button Style"),
    )
    button_size = blocks.ChoiceBlock(
        choices=cms_settings.CJKCMS_FRONTEND_BTN_SIZE_CHOICES,
        default=cms_settings.CJKCMS_FRONTEND_BTN_SIZE_DEFAULT,
        required=False,
        label=_("Button Size"),
    )
    visible_for = blocks.ChoiceBlock(
        choices=cms_settings.CJKCMS_AUTH_VISIBILITY_CHOICES,
        default=cms_settings.CJKCMS_AUTH_VISIBILITY_DEFAULT,
        required=False,
        label=_("Item visibility"),
    )


class CjkcmsAdvSettings(blocks.StructBlock):
    """
    Common fields each block should have,
    which are hidden under the block's "Advanced Settings" dropdown.
    """

    # placeholder, real value get set in __init__()
    custom_template = blocks.Block()

    custom_css_class = blocks.CharBlock(
        required=False,
        max_length=255,
        label=_("Custom CSS Class"),
    )
    custom_id = blocks.CharBlock(
        required=False,
        max_length=255,
        label=_("Custom ID"),
    )

    class Meta:
        form_template = "wagtailadmin/block_forms/base_block_settings_struct.html"
        label = _("Advanced Settings")

    def __init__(self, local_blocks=None, template_choices=None, **kwargs):
        if not local_blocks:
            local_blocks = ()

        local_blocks += (
            (
                "custom_template",
                blocks.ChoiceBlock(
                    choices=template_choices,
                    default=None,
                    required=False,
                    label=_("Template"),
                ),
            ),
        )

        super().__init__(local_blocks, **kwargs)


class CjkcmsAdvTrackingSettings(CjkcmsAdvSettings):
    """
    CjkcmsAdvSettings plus additional tracking fields.
    """

    ga_tracking_event_category = blocks.CharBlock(
        required=False,
        max_length=255,
        label=_("Tracking Event Category"),
    )
    ga_tracking_event_label = blocks.CharBlock(
        required=False,
        max_length=255,
        label=_("Tracking Event Label"),
    )


class CjkcmsAdvColumnSettings(CjkcmsAdvSettings):
    """
    BaseBlockSettings plus additional column fields.
    """

    column_breakpoint = blocks.ChoiceBlock(
        choices=cms_settings.CJKCMS_FRONTEND_COL_BREAK_CHOICES,
        default=cms_settings.CJKCMS_FRONTEND_COL_BREAK_DEFAULT,
        required=False,
        verbose_name=_("Column Breakpoint"),
        help_text=_(
            "Screen size at which the column will expand horizontally or stack vertically."
        ),  # noqa
    )


class BaseBlock(blocks.StructBlock):
    """
    Common attributes for all blocks used in the CMS.
    """

    # subclasses can override this to determine the advanced settings class
    advsettings_class = CjkcmsAdvSettings

    # placeholder, real value get set in __init__() from advsettings_class
    settings = blocks.Block()

    def __init__(self, local_blocks=None, **kwargs):
        """
        Construct and inject settings block, then initialize normally.
        """
        klassname = self.__class__.__name__.lower()
        choices = cms_settings.CJKCMS_FRONTEND_TEMPLATES_BLOCKS.get(
            "*", []
        ) + cms_settings.CJKCMS_FRONTEND_TEMPLATES_BLOCKS.get(klassname, [])

        if not local_blocks:
            local_blocks = ()

        local_blocks += (
            ("settings", self.advsettings_class(template_choices=choices)),
        )

        super().__init__(local_blocks, **kwargs)

    def render(self, value, context=None):
        template = value["settings"]["custom_template"]
        if not template:
            template = self.get_template(context=context)
        if not template:
            return self.render_basic(value, context=context)
        if context is None:
            new_context = self.get_context(value)
        else:
            new_context = self.get_context(value, parent_context=dict(context))
        return mark_safe(render_to_string(template, new_context))


class BaseLayoutBlock(BaseBlock):
    """
    Common attributes for all blocks used in the CMS.
    """

    # Subclasses can override this to provide a default list of blocks for the content.
    content_streamblocks = []

    def __init__(self, local_blocks=None, **kwargs):
        if not local_blocks and self.content_streamblocks:
            local_blocks = self.content_streamblocks

        if local_blocks:
            local_blocks = (
                ("content", blocks.StreamBlock(local_blocks, label=_("Content"))),
            )

        super().__init__(local_blocks, **kwargs)


class LinkStructValue(blocks.StructValue):
    """
    Generates a URL for blocks with multiple link choices.
    """

    @property
    def url(self):
        page = self.get("page_link")
        doc = self.get("doc_link")
        ext = self.get("other_link")
        if page and ext:
            return "{0}{1}".format(page.url, ext)
        elif page:
            return page.url
        elif doc:
            return doc.url
        else:
            return ext


class BaseLinkBlock(BaseBlock):
    """
    Common attributes for creating a link within the CMS.
    """

    page_link = blocks.PageChooserBlock(
        required=False,
        label=_("Page link"),
    )
    doc_link = DocumentChooserBlock(
        required=False,
        label=_("Document link"),
    )
    other_link = blocks.CharBlock(
        required=False,
        max_length=255,
        label=_("Other link"),
    )

    advsettings_class = CjkcmsAdvTrackingSettings

    class Meta:
        value_class = LinkStructValue



from wagtail.core.models import Site
from cjkcms.models.wagtailsettings_models import MailchimpApiSettings

import requests


class MailchimpApi:
    user_string = "Website"
    proto_base_url = "https://{0}.api.mailchimp.com/3.0/"

    def __init__(self, site=None):
        self.set_access_token(site)

    def set_access_token(self, site=None):
        site = site or Site.objects.get(is_default_site=True)
        self.access_token = MailchimpApiSettings.for_site(site).mailchimp_api_key
        if self.access_token:
            self.set_base_url()
            self.is_active = True
        else:
            self.is_active = False

    def set_base_url(self):
        """
        The base url for the mailchimip api is dependent on the api key.
        """
        key, datacenter = self.access_token.split("-")
        self.base_url = self.proto_base_url.format(datacenter)

    def default_headers(self):
        return {
            "Content-Type": "application/json",
        }

    def default_auth(self):
        return self.user_string, self.access_token

    def get_lists(self):
        endpoint = "lists?fields=lists.name,lists.id"
        return self._get(endpoint)

    def get_merge_fields_for_list(self, list_id):
        endpoint = (
            f"lists/{list_id}/merge-fields"
            "?fields=merge_fields.tag,merge_fields.merge_id,merge_fields.name"
        )
        return self._get(endpoint)

    def get_interest_categories_for_list(self, list_id):
        endpoint = (
            f"lists/{list_id}/interest-categories?fields=categories.id,categories.title"
        )
        return self._get(endpoint)

    def get_interests_for_interest_category(self, list_id, interest_category_id):
        endpoint = (
            f"lists/{list_id}/interest-categories/{interest_category_id}/interests"
            "?fields=interests.id,interests.name"
        )
        return self._get(endpoint)

    def add_user_to_list(self, list_id, data):
        endpoint = "lists/{0}".format(list_id)
        return self._post(endpoint, data=data)

    def _get(self, endpoint, data=None, auth=None, headers=None, **kwargs):
        if data is None:
            data = {}
        auth = auth or self.default_auth()
        headers = headers or self.default_headers()
        full_url = "{0}{1}".format(self.base_url, endpoint)
        r = requests.get(full_url, auth=auth, headers=headers, data=data, **kwargs)
        return r.json()

    def _post(self, endpoint, data=None, auth=None, headers=None, **kwargs):
        if data is None:
            data = {}
        auth = auth or self.default_auth()
        headers = headers or self.default_headers()
        full_url = "{0}{1}".format(self.base_url, endpoint)
        r = requests.post(full_url, auth=auth, headers=headers, data=data, **kwargs)
        return r.json()



from django.db import models
from django.forms.widgets import Textarea
from wagtail.fields import StreamField

from .widgets import ColorPickerWidget


class CjkcmsStreamField(StreamField):
    """
    An exact copy of the Wagtail StreamField, modified to NOT PRESERVE HISTORY
    in Django migrations.

    Since our StreamFields are generally huge, and we also let sites override
    the blocks in our concrete models dynamically, this creates a slew of
    migration problems (most commonly: a client overrides CJKCMS_FRONTEND_*,
    which changes a string used in a concrete model, which triggers a migration
    back in coderedcms). Eliminiating the blocks from the deconstructed
    StreamField allows us to have dynamic streamfields without breaking
    migrations or having to refactor the core concepts of this package.

    Internally, we should ALWAYS use CoderedStreamField on CONCRETE models,
    meaning models which are part of our package and saved to the database. For
    ABSTRACT models - meaning they are made concrete in the client site - we
    should continue to use Wagtail StreamField to keep things Wagtail-ish by
    default. The client may then decide to use CoderedStreamField if they are
    annoyed by the big migrations.

    CAVEAT EMPTOR:

    Client sites built with the CMS may use this in place of the Wagtail
    StreamField, in order to avoid huge migration files. However, note that it
    will not be possible to mine data out of a CjkcmsStreamField during a
    migration (e.g. RunPython).

    Inspired by:
    https://cynthiakiser.com/blog/2022/01/06/trimming-wagtail-migration-cruft.html
    """

    def __init__(self, *args, **kwargs):
        """
        Patch init to work around django reconstruct not sending empty args.
        """
        # If we did not get an arg, pass an empty list through to the parent.
        if not args:
            args = [[]]
        return super().__init__(*args, **kwargs)

    def deconstruct(self):
        """
        Override to ignore any blocks within the StreamField when
        decustructing into a migration.

        The output should look something like this, regardless of how many
        blocks are nested within the StreamField::

            ("body", cjkcms.fields.CjkcmsStreamField([]))

        """
        name, path, block_types, kwargs = super().deconstruct()
        block_types = []
        return name, path, block_types, kwargs


class ColorField(models.CharField):
    """
    A CharField which uses the HTML5 color picker widget.
    """

    def __init__(self, *args, **kwargs):
        kwargs["max_length"] = 255
        super().__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        kwargs["widget"] = ColorPickerWidget
        return super().formfield(**kwargs)


class MonospaceField(models.TextField):
    """
    A TextField which renders as a large HTML textarea with monospace font.
    """

    def formfield(self, **kwargs):
        kwargs["widget"] = Textarea(
            attrs={
                "rows": 12,
                "class": "monospace",
                "spellcheck": "false",
            }
        )
        return super().formfield(**kwargs)



import hashlib
import urllib.parse

from django.template import Library
from wagtail.core.models import Site

from cjkcms.models import GeneralSettings

register = Library()


@register.simple_tag
def gravatar_url(email, size=80, default_url=None):
    """
    Returns gravatar url for:
    - email
    - with a specific image size
    - with a default image url, if email-hash not found in gravatar db
    """
    if default_url:
        d_url = default_url
    else:
        site = Site.objects.filter(is_default_site=True)[0]
        d_gravatar = GeneralSettings.for_site(site).gravatar
        d_url = d_gravatar.get_rendition("original").url if d_gravatar else ""

    gravatar_url = (
        "https://www.gravatar.com/avatar/"
        + hashlib.md5(email.lower().encode("utf-8")).hexdigest()
        + "?"
    )
    gravatar_url += urllib.parse.urlencode({"s": str(size), "d": d_url})
    return gravatar_url


@register.simple_tag
def gravatar(email, size=80, default_url=None, options=""):
    url = gravatar_url(email, size, default_url)
    return '<img src="%s" width="%s" height="%s" %s >' % (url, size, size, options)



# Generated by Django 4.0.7 on 2022-09-02 22:15

import cjkcms.fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('cjkcms', '0002_alter_body_to_cjkcmsstreamfield'),
    ]

    operations = [
        migrations.AlterField(
            model_name='footer',
            name='content',
            field=cjkcms.fields.CjkcmsStreamField(use_json_field=True, verbose_name='Content'),
        ),
        migrations.AlterField(
            model_name='navbar',
            name='menu_items',
            field=cjkcms.fields.CjkcmsStreamField(use_json_field=True, verbose_name='Navigation links'),
        ),
    ]



"""
Blocks module entry point. Used to cleanly organize blocks into
individual files based on purpose, but provide them all as a
single `blocks` module.
"""

from django.utils.translation import gettext_lazy as _

from .html_blocks import (
    ButtonBlock,
    ImageBlock,
    ImageLinkBlock,
    DownloadBlock,
    EmbedVideoBlock,
    PageListBlock,
    PagePreviewBlock,
    QuoteBlock,
    RichTextBlock,
    TableBlock,
    SearchableHTMLBlock,
)
from .content_blocks import (  # noqa
    AccordionBlock,
    CardBlock,
    CarouselBlock,
    ImageGalleryBlock,
    ModalBlock,
    NavDocumentLinkWithSubLinkBlock,
    NavExternalLinkWithSubLinkBlock,
    NavPageLinkWithSubLinkBlock,
    PriceListBlock,
    ReusableContentBlock,
)
from .layout_blocks import CardGridBlock, GridBlock, HeroBlock
from cjkcms.settings import cms_settings

# Collections of blocks commonly used together.

HTML_STREAMBLOCKS = [
    (
        "text",
        RichTextBlock(
            icon="font",
            features=cms_settings.CJKCMS_RICHTEXT_FEATURES["full"],
        ),
    ),
    ("button", ButtonBlock()),
    ("image", ImageBlock()),
    ("image_link", ImageLinkBlock()),
    (
        "html",
        SearchableHTMLBlock(
            icon="code",
            form_classname="monospace",
            label=_("HTML"),
        ),
    ),
    ("download", DownloadBlock()),
    ("embed_video", EmbedVideoBlock()),
    ("quote", QuoteBlock()),
    ("table", TableBlock()),
    ("page_list", PageListBlock()),
    ("page_preview", PagePreviewBlock()),
]

CONTENT_STREAMBLOCKS = HTML_STREAMBLOCKS + [
    ("accordion", AccordionBlock()),
    ("card", CardBlock()),
    ("carousel", CarouselBlock()),
    ("image_gallery", ImageGalleryBlock()),
    ("modal", ModalBlock(HTML_STREAMBLOCKS)),
    ("pricelist", PriceListBlock()),
    ("reusable_content", ReusableContentBlock()),
]

NAVIGATION_STREAMBLOCKS = [
    ("page_link", NavPageLinkWithSubLinkBlock()),
    ("external_link", NavExternalLinkWithSubLinkBlock()),
    ("document_link", NavDocumentLinkWithSubLinkBlock()),
]

BASIC_LAYOUT_STREAMBLOCKS = [
    ("row", GridBlock(HTML_STREAMBLOCKS)),
    (
        "html",
        SearchableHTMLBlock(icon="code", form_classname="monospace", label=_("HTML")),
    ),
]

LAYOUT_STREAMBLOCKS = [
    (
        "hero",
        HeroBlock(
            [
                ("row", GridBlock(CONTENT_STREAMBLOCKS)),
                (
                    "cardgrid",
                    CardGridBlock(
                        [
                            ("card", CardBlock()),
                        ]
                    ),
                ),
                (
                    "html",
                    SearchableHTMLBlock(
                        icon="code", form_classname="monospace", label=_("HTML")
                    ),
                ),
            ]
        ),
    ),
    ("row", GridBlock(CONTENT_STREAMBLOCKS)),
    (
        "cardgrid",
        CardGridBlock(
            [
                ("card", CardBlock()),
            ]
        ),
    ),
    (
        "html",
        SearchableHTMLBlock(icon="code", form_classname="monospace", label=_("HTML")),
    ),
]



import re

from django.template import engines
from django.test import TestCase
from wagtail.core.models import Site
from cjkcms.models import AdobeApiSettings


django_engine = engines["django"]
html_id_re = re.compile(r"^[A-Za-z][A-Za-z0-9_:.-]*$")

version_re = re.compile(
    r"^(\d+!)?(\d+)(\.\d+)+([\.\-\_])?((a(lpha)?|b(eta)?|c|r(c|ev)?|pre(view)?)\d*)?(\.?(post|dev)\d*)?$"  # noqa: E501
)


class TemplateTagTests(TestCase):
    def test_cjkcms_generate_random_id(self):
        count = 1000
        t = django_engine.from_string(
            "{% load cjkcms_tags %}{% generate_random_id as rid %}{{rid}}"
        )

        ids = set([])
        for _ in range(count):
            ids.add(t.render(None))
        self.assertEqual(len(ids), count)
        for i, value in enumerate(ids, start=1):
            self.assertTrue(
                html_id_re.match(value),
                'ID #%s "%s" did not match regex %r' % (i, value, html_id_re),
            )

    def test_cjkcms_version(self):
        rt = django_engine.from_string(
            "{% load cjkcms_tags %}{% cjkcms_version %}"
        ).render(None)
        self.assertTrue(
            version_re.match(rt),
            "App Version format incorrect: %r" % version_re,
        )

    def test_django_settings_filter(self):
        rt = django_engine.from_string(
            "{% load cjkcms_tags %}{{ 'APPEND_SLASH'|django_settings  }}"
        ).render(None)
        self.assertIn(
            rt,
            ["True", "False"],
            f"Django setting APPEND_SLASH is neither True nor False, instead is {rt}",
        )

    def test_map_to_bootstrap_alert_filter(self):
        rt = django_engine.from_string(
            "{% load cjkcms_tags %}{{ 'error'|map_to_bootstrap_alert }}"
        ).render(None)
        self.assertEqual(
            rt, "danger", "map_to_bootstrap_alert for 'error' did not return 'danger'"
        )

    def test_brand_logo_long_tag(self):
        rt = django_engine.from_string(
            "{% load cjkcms_tags %}{% brand_logo_long %}"
        ).render(None)
        self.assertIn(
            rt[-4:],
            [".png", ".jpg", ".webp", ".svg"],
            f"Django setting BRAND_LOGO_LONG does not seem to return one of [png,jpg,webp,svg]: {rt}",  # noqa: E501
        )

    def test_brand_logo_square_tag(self):
        rt = django_engine.from_string(
            "{% load cjkcms_tags %}{% brand_logo_square %}"
        ).render(None)
        self.assertIn(
            rt[-4:],
            [".png", ".jpg", ".webp", ".svg"],
            f"Django setting BRAND_LOGO_LONG does not seem to return one of [png,jpg,webp,svg]: {rt}",  # noqa: E501
        )

    def test_AdobeApiKeyInTemplate(self):
        site = Site.objects.filter(is_default_site=True)[0]
        adobe_api_key = AdobeApiSettings.for_site(site=site)
        adobe_api_key.adobe_embed_id = "test_key"
        adobe_api_key.save()

        rt = django_engine.from_string(
            "{% load wagtailsettings_tags %}{% get_settings use_default_site=True %}{{ settings.cjkcms.AdobeApiSettings.adobe_embed_id }}"  # noqa: E501
        ).render(None)
        self.assertEqual(
            rt, "test_key", "Adobe API key not returned in template context"
        )



from django import forms


class ColorPickerWidget(forms.TextInput):
    input_type = "color"


class ClassifierSelectWidget(forms.CheckboxSelectMultiple):
    template_name = "cjkcms/widgets/checkbox_classifiers.html"

    def optgroups(self, name, value, attrs=None):
        # sourcery skip: for-append-to-extend, list-comprehension
        from cjkcms.models.snippet_models import Classifier

        classifiers = Classifier.objects.all().select_related()

        groups = []
        has_selected = False

        for index, classifier in enumerate(classifiers):
            subgroup = []
            group_name = classifier.name
            subindex = 0
            choices = []

            for term in classifier.terms.all():
                choices.append((term.pk, term.name))

            groups.append((group_name, subgroup, index))

            for subvalue, sublabel in choices:
                selected = str(subvalue) in value and (
                    not has_selected or self.allow_multiple_selected
                )
                has_selected |= selected
                subgroup.append(
                    self.create_option(
                        name,
                        subvalue,
                        sublabel,
                        selected,
                        index,
                        subindex=subindex,
                        attrs=attrs,
                    )
                )
                if subindex is not None:
                    subindex += 1
        return groups



from django import template
from typing import TypeVar

register = template.Library()
T = TypeVar("T")


@register.tag(name="captureas")
def do_captureas(parser, token):
    """
    Capture content for re-use throughout a template.
    particularly handy for use within social meta fields
    that are virtually identical.
    """
    try:
        tag_name, args = token.contents.split(None, 1)
    except ValueError as e:
        raise template.TemplateSyntaxError(
            "'captureas' node requires a variable name."
        ) from e

    nodelist = parser.parse(("endcaptureas",))
    parser.delete_first_token()
    return CaptureasNode(nodelist, args)


class CaptureasNode(template.Node):
    def __init__(self, nodelist, varname):
        self.nodelist = nodelist
        self.varname = varname

    def render(self, context):
        output = self.nodelist.render(context)
        context[self.varname] = output
        return ""


@register.simple_tag
def assign(variable: T) -> T:
    return variable


@register.filter
def in_list(value, the_list):
    """
    Returns true if value is in comma-separated list built from the_list string
    """
    value = str(value)
    return value in the_list.split(",")



from django.core.management.base import BaseCommand
from wagtail.core.models import Collection


class Command(BaseCommand):
    """Create a set of commonly used collections for a new website:
    covers, logos, resources, people, articles, cards, icons"""

    help = "Create a set of commonly used collections: "
    "covers, logos, resources, people, articles, cards, icons"

    def handle(self, *args, **options):
        names = ["Covers", "Logos", "Resources", "People", "Articles", "Cards", "Icons"]

        # try creating collection, if does not exist

        root_coll = Collection.get_first_root_node()

        if not root_coll:

            return

        for n in names:
            if Collection.objects.filter(name=n).exists():
                self.stdout.write(f"Collection {n} already exists, skipping creation.")
                continue
            root_coll.add_child(name=n)
            self.stdout.write(f"Created collection {n}")



from cjkcms.forms import SearchForm
from cjkcms.models import (
    GeneralSettings,
    LayoutSettings,
)

from django.http import Http404, HttpResponsePermanentRedirect
from django.contrib.contenttypes.models import ContentType
from django.core.paginator import Paginator, InvalidPage, EmptyPage, PageNotAnInteger
from django.shortcuts import render
from wagtail.models import Page, get_page_models

# from coderedcms.importexport import convert_csv_to_json, import_pages, ImportPagesFromCSVFileForm
from cjkcms.templatetags.cjkcms_tags import get_name_of_class


def search(request):
    """
    Searches pages across the entire site.
    """
    search_form = SearchForm(request.GET)
    pagetypes = []
    results = None
    results_paginated = None

    if search_form.is_valid():
        search_query = search_form.cleaned_data["s"]
        search_model = search_form.cleaned_data["t"]

        # get all page models
        pagemodels = sorted(get_page_models(), key=get_name_of_class)
        # filter based on is search_filterable
        for model in pagemodels:
            if hasattr(model, "search_filterable") and model.search_filterable:
                pagetypes.append(model)

        results = Page.objects.live()
        if search_model:
            try:
                # If provided a model name, try to get it
                model = ContentType.objects.get(model=search_model).model_class()
                results = results.type(model)
            except ContentType.DoesNotExist:
                # Maintain existing behavior of only returning objects if the page type is real
                results = None

        # get and paginate results
        if results:
            results = results.search(search_query)
            paginator = Paginator(
                results, GeneralSettings.for_request(request).search_num_results
            )
            page = request.GET.get("p", 1)
            try:
                results_paginated = paginator.page(page)
            except PageNotAnInteger:
                results_paginated = paginator.page(1)
            except EmptyPage:
                results_paginated = paginator.page(1)
            except InvalidPage:
                results_paginated = paginator.page(1)

    # Render template
    return render(
        request,
        "cjkcms/pages/search.html",
        {
            "request": request,
            "pagetypes": pagetypes,
            "form": search_form,
            "results": results,
            "results_paginated": results_paginated,
        },
    )


def favicon(request):
    if icon := LayoutSettings.for_request(request).favicon:
        return HttpResponsePermanentRedirect(icon.get_rendition("original").url)
    raise Http404()


def robots(request):
    return render(request, "cjkcms/robots.txt", content_type="text/plain")


# @login_required
# def import_index(request):
#     """
#     Landing page to replace wagtailimportexport.
#     """
#     return render(request, 'wagtailimportexport/index.html')


# @login_required
# def import_pages_from_csv_file(request):
#     """
#     Overwrite of the `import_pages` view from wagtailimportexport.  By default, the `import_pages`
#     view expects a json file to be uploaded.  This view converts the uploaded csv into the json
#     format that the importer expects.
#     """

#     if request.method == 'POST':
#         form = ImportPagesFromCSVFileForm(request.POST, request.FILES)
#         if form.is_valid():
#             import_data = convert_csv_to_json(
#                 form.cleaned_data['file'].read().decode('utf-8').splitlines(),
#                 form.cleaned_data['page_type']
#             )
#             parent_page = form.cleaned_data['parent_page']
#             try:
#                 page_count = import_pages(import_data, parent_page)
#             except LookupError as e:
#                 messages.error(request, _(
#                     "Import failed: %(reason)s") % {'reason': e}
#                 )
#             else:
#                 messages.success(request, ngettext(
#                     "%(count)s page imported.",
#                     "%(count)s pages imported.",
#                     page_count) % {'count': page_count}
#                 )
#             return redirect('wagtailadmin_explore', parent_page.pk)
#     else:
#         form = ImportPagesFromCSVFileForm()

#     return render(request, 'wagtailimportexport/import_from_csv.html', {
#         'form': form,
#     })



import mimetypes

from django.templatetags.static import static
from django.http.response import HttpResponse
from django.utils.html import format_html
from wagtail.admin.menu import MenuItem
from wagtail import hooks
from wagtailcache.cache import clear_cache

from cjkcms import __version__
from cjkcms.draftail import (
    register_inline_styling,
    register_block_feature,
    DRAFTAIL_ICONS,
)


@hooks.register("insert_global_admin_css")
def global_admin_css():
    return format_html(
        '<link rel="stylesheet" type="text/css" href="{}?v={}">',
        static("cjkcms/css/cjkcms-admin.css"),
        __version__,
    )


@hooks.register("insert_editor_css")
def editor_css():
    return format_html(
        '<link rel="stylesheet" type="text/css" href="{}?v={}">',
        static("cjkcms/css/cjkcms-editor.css"),
        __version__,
    )


@hooks.register("insert_editor_js")
def collapsible_js():
    return format_html(
        '<script src="{}?v={}"></script>',
        static("cjkcms/js/cjkcms-editor.js"),
        __version__,
    )


@hooks.register("register_icons")
def register_icons(icons):
    """
    Add custom SVG icons to the Wagtail admin.
    """
    # These SVG files should be in the django templates folder, and follow exact
    # specifications to work with Wagtail:
    # https://github.com/wagtail/wagtail/pull/6028
    icons.append("cjkcms/icons/align-left.svg")
    icons.append("cjkcms/icons/check-square-o.svg")
    icons.append("cjkcms/icons/columns.svg")
    icons.append("cjkcms/icons/desktop.svg")
    icons.append("cjkcms/icons/font.svg")
    icons.append("cjkcms/icons/google.svg")
    icons.append("cjkcms/icons/hand-pointer-o.svg")
    icons.append("cjkcms/icons/hashtag.svg")
    icons.append("cjkcms/icons/header.svg")
    icons.append("cjkcms/icons/list-alt.svg")
    icons.append("cjkcms/icons/map.svg")
    icons.append("cjkcms/icons/newspaper-o.svg")
    icons.append("cjkcms/icons/puzzle-piece.svg")
    icons.append("cjkcms/icons/recycle.svg")
    icons.append("cjkcms/icons/stop.svg")
    icons.append("cjkcms/icons/th-large.svg")
    icons.append("cjkcms/icons/universal-access.svg")
    icons.append("cjkcms/icons/usd.svg")
    icons.append("cjkcms/icons/window-maximize.svg")
    icons.append("cjkcms/icons/window-minimize.svg")
    return icons


def clear_wagtailcache(*args, **kwargs):
    clear_cache()


# Clear cache whenever pages/snippets are changed. Err on the side of clearing
# the cache vs not clearing the cache, as this usually leads to support requests
# when staff members make edits but do not see the changes.
hooks.register("after_delete_page", clear_wagtailcache)
hooks.register("after_move_page", clear_wagtailcache)
hooks.register("after_publish_page", clear_wagtailcache)
hooks.register("after_unpublish_page", clear_wagtailcache)
hooks.register("after_create_snippet", clear_wagtailcache)
hooks.register("after_edit_snippet", clear_wagtailcache)
hooks.register("after_delete_snippet", clear_wagtailcache)


@hooks.register("before_serve_document")
def serve_document_directly(document, request):
    """
    This hook prevents documents from being downloaded unless
    specified by an <a> tag with the download attribute.
    """
    content_type, content_encoding = mimetypes.guess_type(document.filename)
    response = HttpResponse(document.file.read(), content_type=content_type)
    response["Content-Disposition"] = 'inline;filename="{0}"'.format(document.filename)
    response["Content-Encoding"] = str(content_encoding)
    return response


class ImportExportMenuItem(MenuItem):
    def is_shown(self, request):
        return request.user.is_superuser


# @hooks.register("register_settings_menu_item")
# def register_import_export_menu_item():
#     return ImportExportMenuItem(
#         _("Import"),
#         reverse("import_index"),
#         classnames="icon icon-download",
#     )


@hooks.register("register_rich_text_features")
def register_underline_styling(features):
    register_inline_styling(
        features=features,
        feature_name="underline",
        type_="UNDERLINE",
        tag="u",
        description="Underline",
        label="U̲",
    )


@hooks.register("register_rich_text_features")
def register_larger_styling(features):
    register_inline_styling(
        features=features,
        feature_name="larger",
        type_="LARGER",
        tag="span",
        format='style="font-size:larger"',
        editor_style={"font-size": "larger"},
        description="Increase Font",
        icon=DRAFTAIL_ICONS.increase_font,
    )


@hooks.register("register_rich_text_features")
def register_smaller_styling(features):
    register_inline_styling(
        features=features,
        feature_name="smaller",
        type_="SMALLER",
        tag="span",
        format='style="font-size:smaller"',
        editor_style={"font-size": "smaller"},
        description="Decrease Font",
        icon=DRAFTAIL_ICONS.decrease_font,
    )


@hooks.register("register_rich_text_features")
def register_align_left_feature(features):
    register_block_feature(
        features=features,
        feature_name="left-align",
        type_="LEFT-ALIGN",
        description="Left align text",
        css_class="text-start",
        element="p",
        icon=DRAFTAIL_ICONS.left_align,
    )


@hooks.register("register_rich_text_features")
def register_align_centre_feature(features):
    register_block_feature(
        features=features,
        feature_name="centre-align",
        type_="CENTRE-ALIGN",
        description="Centre align text",
        css_class="text-center",
        element="p",
        icon=DRAFTAIL_ICONS.centre_align,
    )


@hooks.register("register_rich_text_features")
def register_align_right_feature(features):
    register_block_feature(
        features=features,
        feature_name="right-align",
        type_="RIGHT-ALIGN",
        description="Right align text",
        css_class="text-end",
        element="p",
        icon=DRAFTAIL_ICONS.right_align,
    )
